@@ -48,7 +48,7 @@ pca_mfd <- function(mfdobj, scale = TRUE, nharm = 20) {
   nbasis <- mfdobj$basis$nbasis
   nharm <- min(nobs - 1, nvar * nbasis, nharm)
 
-  pca <- pca.fd(data_pca, nharm = nharm, centerfns = FALSE)
+  pca <- pca.fd_inprods_faster(data_pca, nharm = nharm, centerfns = FALSE)
   pca$harmonics$fdnames[c(1, 3)] <- mfdobj$fdnames[c(1, 3)]
 
   pca$pcscores <- if(length(dim(pca$scores)) == 3) {
@@ -91,8 +91,7 @@ pca_mfd <- function(mfdobj, scale = TRUE, nharm = 20) {
   pca$harmonics <- mfd(coefs,
                        pca$harmonics$basis,
                        pca$harmonics$fdnames,
-                       NULL,
-                       NULL)
+                       B = mfdobj$basis$B)
 
   class(pca) <- c("pca_mfd", "pca.fd")
   pca
@@ -241,6 +240,9 @@ get_scores <- function(pca, components, newdata_scaled = NULL) {
 #'
 get_pre_scores <- function(pca, components, newdata_scaled = NULL) {
 
+  if (missing(components)) {
+    stop("components argument must be provided")
+  }
   if (is.null(newdata_scaled)) {
     inprods <- pca$scores[, components, , drop = FALSE]
   } else {
@@ -350,7 +352,7 @@ get_fit_pca_given_scores <- function(scores, harmonics) {
   fdnames <- list(harmonics$fdnames[[1]],
                   obs, variables)
 
-  mfd(fit_coefs, basis, fdnames)
+  mfd(fit_coefs, basis, fdnames, B = basis$B)
 }
 
 #' Calculate the Hotelling's T^2 statistics of multivariate functional data
@@ -417,7 +419,7 @@ get_T2_spe <- function(pca, components, newdata_scaled = NULL) {
     minus.fd(newdata_scaled, fit)
   }
 
-  res_fd <- mfd(res_fd$coefs, res_fd$basis, res_fd$fdnames)
+  res_fd <- mfd(res_fd$coefs, res_fd$basis, res_fd$fdnames, B = res_fd$basis$B)
 
   cont_spe <- inprod_mfd_diag(res_fd)
   rownames(cont_spe) <- obs
@@ -430,3 +432,136 @@ get_T2_spe <- function(pca, components, newdata_scaled = NULL) {
   cbind(out_T2, out_spe)
 
 }
+
+
+#' @noRd
+#'
+pca.fd_inprods_faster <- function (fdobj,
+                                   nharm = 2,
+                                   harmfdPar = fdPar(fdobj),
+                                   centerfns = TRUE) {
+  if (!(is.fd(fdobj) || is.fdPar(fdobj)))
+    stop("First argument is neither a functional data or a functional parameter object.")
+  if (is.fdPar(fdobj))
+    fdobj <- fdobj$fd
+
+  if (length(dim(fdobj$coefs)) == 3 & dim(fdobj$coefs)[3] == 1) {
+    fdobj$coefs <- matrix(fdobj$coefs,
+                          nrow = dim(fdobj$coefs)[1],
+                          ncol = dim(fdobj$coefs)[2])
+  }
+
+  meanfd <- mean.fd(fdobj)
+  if (centerfns) {
+    fdobj <- center.fd(fdobj)
+  }
+  coef <- fdobj$coefs
+  coefd <- dim(coef)
+  ndim <- length(coefd)
+  nrep <- coefd[2]
+  coefnames <- dimnames(coef)
+  if (nrep < 2)
+    stop("PCA not possible without replications.")
+  basisobj <- fdobj$basis
+  nbasis <- basisobj$nbasis
+  type <- basisobj$type
+  harmbasis <- harmfdPar$fd$basis
+  nhbasis <- harmbasis$nbasis
+  Lfdobj <- harmfdPar$Lfd
+  lambda <- harmfdPar$lambda
+
+  if (ndim == 3) {
+    nvar <- coefd[3]
+    ctemp <- matrix(0, nvar * nbasis, nrep)
+    for (j in 1:nvar) {
+      index <- 1:nbasis + (j - 1) * nbasis
+      ctemp[index, ] <- coef[, , j]
+    }
+  }
+  else {
+    nvar <- 1
+    ctemp <- coef
+  }
+  Lmat <- eval.penalty(harmbasis, 0)
+  if (lambda > 0) {
+    Rmat <- eval.penalty(harmbasis, Lfdobj)
+    Lmat <- Lmat + lambda * Rmat
+  }
+  Lmat <- (Lmat + t(Lmat))/2
+  Mmat <- chol(Lmat)
+  Mmatinv <- solve(Mmat)
+  Wmat <- crossprod(t(ctemp))/nrep
+  if (identical(harmbasis, basisobj)) {
+    Jmat <- inprod.bspline(fd(diag(harmbasis$nbasis), harmbasis))
+  } else Jmat = inprod.bspline(fd(diag(harmbasis$nbasis), harmbasis),
+                               fd(diag(harmbasis$nbasis), basisobj))
+  MIJW = crossprod(Mmatinv, Jmat)
+  if (nvar == 1) {
+    Cmat = MIJW %*% Wmat %*% t(MIJW)
+  }
+  else {
+    Cmat = matrix(0, nvar * nhbasis, nvar * nhbasis)
+    for (i in 1:nvar) {
+      indexi <- 1:nbasis + (i - 1) * nbasis
+      for (j in 1:nvar) {
+        indexj <- 1:nbasis + (j - 1) * nbasis
+        Cmat[indexi, indexj] <- MIJW %*% Wmat[indexi,
+                                              indexj] %*% t(MIJW)
+      }
+    }
+  }
+  Cmat <- (Cmat + t(Cmat))/2
+  result <- eigen(Cmat)
+  eigvalc <- result$values
+  eigvecc <- as.matrix(result$vectors[, 1:nharm])
+  sumvecc <- apply(eigvecc, 2, sum)
+  eigvecc[, sumvecc < 0] <- -eigvecc[, sumvecc < 0]
+  varprop <- eigvalc[1:nharm]/sum(eigvalc)
+  if (nvar == 1) {
+    harmcoef <- Mmatinv %*% eigvecc
+  }
+  else {
+    harmcoef <- array(0, c(nbasis, nharm, nvar))
+    for (j in 1:nvar) {
+      index <- 1:nbasis + (j - 1) * nbasis
+      temp <- eigvecc[index, ]
+      harmcoef[, , j] <- Mmatinv %*% temp
+    }
+  }
+  harmnames <- rep("", nharm)
+  for (i in 1:nharm) harmnames[i] <- paste("PC", i, sep = "")
+  if (length(coefd) == 2)
+    harmnames <- list(coefnames[[1]], harmnames, "values")
+  if (length(coefd) == 3)
+    harmnames <- list(coefnames[[1]], harmnames, coefnames[[3]])
+  harmfd <- fd(harmcoef, harmbasis, harmnames)
+  if (is.null(fdobj$basis$B)) {
+    B <- inprod.bspline(fd(diag(fdobj$basis$nbasis), fdobj$basis))
+  } else {
+    B <- fdobj$basis$B
+  }
+
+  if (nvar == 1) {
+    if (identical(fdobj$basis, harmfd$basis)) {
+      harmscr <- t(fdobj$coefs) %*% B %*% harmfd$coefs
+    } else harmscr <- inprod(fdobj, harmfd)
+  }
+  else {
+    harmscr <- array(0, c(nrep, nharm, nvar))
+    coefarray <- fdobj$coefs
+    harmcoefarray <- harmfd$coefs
+    for (j in 1:nvar) {
+      fdobjj <- fd(as.matrix(coefarray[, , j]), basisobj)
+      harmfdj <- fd(as.matrix(harmcoefarray[, , j]), basisobj)
+      if (identical(fdobjj$basis, harmfdj$basis)) {
+        harmscr[, , j] <- t(fdobjj$coefs) %*% B %*% harmfdj$coefs
+      } else harmscr[, , j] <- inprod(fdobjj, harmfdj)
+    }
+  }
+  pcafd <- list(harmfd, eigvalc, harmscr, varprop, meanfd)
+  class(pcafd) <- "pca.fd"
+  names(pcafd) <- c("harmonics", "values", "scores", "varprop",
+                    "meanfd")
+  return(pcafd)
+}
+
